@@ -113,7 +113,9 @@ EGLImage createImage(EGLDisplay display, const DmaBufAttributes &dmabufAttribs,
 
     const bool hasModifiers = dmabufAttribs.modifier != DRM_FORMAT_MOD_INVALID;
 
-    QVector<EGLint> attribs;
+    static int lastSize = 37; // 37 is what we get on a normal system with working dmabuf, it's just a reasonable default
+    QList<EGLint> attribs;
+    attribs.reserve(lastSize);
     attribs << EGL_WIDTH << size.width() << EGL_HEIGHT << size.height() << EGL_LINUX_DRM_FOURCC_EXT << EGLint(format)
 
             << EGL_DMA_BUF_PLANE0_FD_EXT << dmabufAttribs.planes[0].fd << EGL_DMA_BUF_PLANE0_OFFSET_EXT << EGLint(dmabufAttribs.planes[0].offset)
@@ -155,6 +157,7 @@ EGLImage createImage(EGLDisplay display, const DmaBufAttributes &dmabufAttribs,
     }
 
     attribs << EGL_NONE;
+    lastSize = attribs.size();
 
     static auto eglCreateImageKHR = (PFNEGLCREATEIMAGEKHRPROC)eglGetProcAddress("eglCreateImageKHR");
 
